<?php 
/*
 * [The BSD License, http://www.opensource.org/licenses/bsd-license.php]
 * Copyright (c) 2009-2011, Sven van der Meer <sven@vandermeer.de>
 * All rights reserved.
 *
 * Redistribution  and  use  in  source  and  binary  forms,  with  or  without
 * modification, are permitted provided that the following conditions are met:
 * 
 *     + Redistributions of source code must retain the above copyright notice,
 *       this list of conditions and the following disclaimer.
 *     + Redistributions  in binary  form must  reproduce the  above copyright
 *       notice, this list  of conditions and  the following disclaimer  in the
 *       documentation and/or other materials provided with the distribution.
 *     + Neither the name of the the author nor the names of its contributors
 *       may be used to endorse or promote products derived from this software
 *       without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS  IS"
 * AND ANY EXPRESS  OR IMPLIED WARRANTIES,  INCLUDING, BUT NOT  LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY  AND FITNESS FOR A  PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN  NO EVENT SHALL  THE COPYRIGHT HOLDER  OR CONTRIBUTORS BE
 * LIABLE  FOR  ANY  DIRECT,  INDIRECT,  INCIDENTAL,  SPECIAL,  EXEMPLARY,   OR
 * CONSEQUENTIAL  DAMAGES  (INCLUDING,  BUT  NOT  LIMITED  TO,  PROCUREMENT  OF
 * SUBSTITUTE GOODS  OR SERVICES;  LOSS OF  USE, DATA,  OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER  CAUSED AND  ON ANY  THEORY OF  LIABILITY, WHETHER  IN
 * CONTRACT,  STRICT LIABILITY,  OR TORT  (INCLUDING NEGLIGENCE  OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE  USE OF THIS SOFTWARE, EVEN IF ADVISED  OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

/**
 * A simple base class for a builder object
 *
 * @package    Classes.SKB
 * @author     Sven van der Meer <sven@vandermeer.de>
 * @copyright  2009-2011 Sven van der Meer
 * @license    http://www.opensource.org/licenses/bsd-license.php  BSD License
 * @version    v1.0.0 build 110901 (01-Sep-11) for PHP v5.3.0
 */
abstract class SKB_Builder{
	/**
	 * The templates linked to the builder
	 * @var SKB_BuilderTemplates
	 */
	protected $templates;


	/**
	 * Class constructor.
	 * 
	 * The class constructor only initialises the local variable templates.
	 */
	public function __construct(){
		$this->templates=new SKB_BuilderTemplates();
	}


	/**
	 * Add CSS and Mime Content Type information to the global SKB configuration.
	 * 
	 * This function is usefull if one wants to manually add CSS files or Mime Content Type information to the global SKB configuration.
	 */
	public function set_header(){
		$this->templates->set_header();
	}


	/**
	 * Execute the Builder's main functionality.
	 * 
	 * This function is called by the reader in the execution state. When you write your own builder,
	 * simply implement the function execute_loop and it is called automatically at the right time.
	 *
	 * @param SKB_Request request the original request object
	 * @param Util_ArBase entries the original array with collected information from the reader
	 */
	public function execute(SKB_Request $request, Util_ArBase $entries){
		$this->execute_loop($request, $entries);
	}


	/**
	 * Execute the Builder's main functionality.
	 * 
	 * This function is called by the builder base class in the execution state. When you write your own builder,
	 * simply implement your functionality here and it is called automatically at the right time.
	 *
	 * @param SKB_Request request the original request object
	 * @param Util_ArBase entries the original array with collected information from the reader
	 */
	abstract public function execute_loop(SKB_Request $request, Util_ArBase $entries);
}
?>